@@ -10,14 +10,13 @@ def gettext(s):
 
 
 def extract(fileobj, keywords, comment_tags, options):
-    '''Babel entry point that extracts translation strings from XML templates.
-    '''
-    from .xml_template import _Parser, _Compiler, expand
+    """Babel entry point that extracts translation strings from XML templates."""
+    from .xml_template import _Parser, _Compiler, _DomTransformer
     source = fileobj.read()
     if isinstance(source, bytes):
         source = source.decode('utf-8')
     doc = _Parser(filename='<string>', source=source).parse()
-    expand(doc)
+    doc = _DomTransformer(doc, strip_text=options.get('strip_text', False)).transform()
     compiler = _Compiler(filename='<string>', doc=doc,
                          mode=options.get('mode', 'xml'),
                          is_fragment=options.get('is_fragment', False))
@@ -25,5 +24,4 @@ def extract(fileobj, keywords, comment_tags, options):
     for node in ir:
         if isinstance(node, TranslatableTextNode):
             if node.text.strip():
-                for line in node.text.split('\n'):
-                    yield (node.lineno, '_', line, [])
+                yield (node.lineno, '_', node.text, [])
\ No newline at end of file
