@@ -35,7 +35,7 @@ class _obj(object):
 class _Template(object):
     """Base Class for all compiled Kajiki Templates.
 
-    All kajiki templates created from an ``ir.TemplateNode`` will
+    All kajiki templates created from a :class:`kajiki.ir.TemplateNode` will
     be subclasses of this class.
 
     As the template body code runs inside ``__main__`` method of this
@@ -91,20 +91,34 @@ class _Template(object):
         self.__globals__.update(context)
 
     def __iter__(self):
-        '''We convert the chunk to string because it can be of any type
+        """We convert the chunk to string because it can be of any type
         -- after all, the template supports expressions such as ${x+y}.
         Here, ``chunk`` can be the computed expression result.
-        '''
+        """
         for chunk in self.__main__():
             yield str(chunk)
 
     def render(self):
+        """Render the template to a string."""
         return ''.join(self)
 
     def _push_with(self, locals_, vars):
+        """Enter a ``py:with`` block.
+
+        When a ``py:with`` block is encountered, previous values
+        of the variables assigned inside the ``py:with`` statement are
+        pushed on top of a stack by :class:`kajiki.ir.WithNode` so that
+        when the node is exited the previous values can be recovered.
+        """
         self._with_stack.append([locals_.get(k, ()) for k in vars])
 
     def _pop_with(self):
+        """Exists a ``py:with`` block.
+
+        When a ``py:with`` block is exited the values stack is popped
+        and the head returned to :class:`kajiki.ir.WithNode` so that
+        it can set any previously existing variable to its old value.
+        """
         return self._with_stack.pop()
 
     def _extend(self, parent):
@@ -143,12 +157,25 @@ class _Template(object):
         return p_inst
 
     def _push_switch(self, expr):
+        """Enter a ``py:switch`` block.
+
+        Pushes provided value on the stack used
+        to check ``py:case`` statements against.
+
+        Calling :meth:`._pop_switch` will exit the switch block.
+        """
         self._switch_stack.append(expr)
 
     def _pop_switch(self):
+        """Exit current ``py:switch`` block.
+
+        Pops current value from the stack used
+        to check ``py:case`` statements against.
+        """
         self._switch_stack.pop()
 
     def _case(self, obj):
+        """Check against current ``py:switch`` value."""
         return obj == self._switch_stack[-1]
 
     def _import(self, name, alias, gbls):
@@ -159,7 +186,7 @@ class _Template(object):
         return r
 
     def _escape(self, value):
-        "Returns the given HTML with ampersands, carets and quotes encoded."
+        """Returns the given HTML with ampersands, carets and quotes encoded."""
         if value is None or isinstance(value, flattener):
             return value
         if hasattr(value, '__html__'):
@@ -180,6 +207,12 @@ class _Template(object):
     _re_escape = re.compile(r'&|<|>|"')
 
     def _render_attrs(self, attrs, mode):
+        """Render tag attributes in key="value" format.
+
+        A :class:`kajiki.ir.AttrsNode` will generate
+        code that in fact leads to this function to generate
+        the html for tag attributes.
+        """
         if hasattr(attrs, 'items'):
             attrs = attrs.items()
         if attrs is not None:
@@ -198,7 +231,10 @@ class _Template(object):
         for part in it:
             if part is None:
                 continue
-            result.append(str(part))
+            if isinstance(part, flattener):
+                result.append(str(part.accumulate_str()))
+            else:
+                result.append(str(part))
         if result:
             return ''.join(result)
         else:
@@ -210,13 +246,14 @@ class _Template(object):
             meth.annotate_lnotab(cls.filename, py_to_tpl, dict(py_to_tpl))
 
     def defined(self, name):
+        """Check if a variable was provided to the template or not"""
         return name in self._context
 
 
 def Template(ns):
-    """Creates a new ``_Template`` subclass from an entity with ``exposed`` functions.
+    """Creates a new :class:`._Template` subclass from an entity with ``exposed`` functions.
 
-    Kajiki used classes as containers of the exposed functions for convenience,
+    Kajiki uses classes as containers of the exposed functions for convenience,
     but any object that can have the functions as attributes works.
 
     To be a valid template the original entity must provide at least a ``__main__``
@@ -245,9 +282,9 @@ def Template(ns):
 def from_ir(ir_node):
     """Creates a template class from Intermediate Representation TemplateNode.
 
-    This actually creates the class defined by the TemplateNode and returns
-    a subclass of it.
-    The returned class is a subclass of a `kajiki.template._Template`.
+    This actually creates the class defined by the TemplateNode by executing
+    its code and returns a subclass of it.
+    The returned class is a subclass of :class:`kajiki.template._Template`.
     """
     py_lines = list(generate_python(ir_node))
     py_text = '\n'.join(map(str, py_lines))
@@ -271,6 +308,15 @@ def from_ir(ir_node):
 
 
 class TplFunc(object):
+    """A template function attached to a _Template.
+
+    By default template functions (ie: __main__) depends
+    on variables like ``self``, ``local`` and so on which
+    are provided by :class:`._Template`.
+
+    This is used by :meth:`.Template` to create a new
+    ``_Template`` with the attached functions.
+    """
     def __init__(self, func, inst=None):
         self._func = func
         self._inst = inst
