@@ -7,7 +7,11 @@ import os
 import sys
 import traceback
 import xml.dom.minidom
+from io import BytesIO
 from unittest import TestCase, main
+
+from kajiki import i18n
+from kajiki.template import KajikiSyntaxError
 from nine import chr, str
 import kajiki
 from kajiki import MockLoader, XMLTemplate, FileLoader, PackageLoader
@@ -48,7 +52,7 @@ class TestExpand(TestCase):
         py:replace="replace"
         py:block="block"
         py:extends="extends">Foo</div>''').parse()
-        kajiki.xml_template.expand(doc)
+        doc = kajiki.xml_template._DomTransformer(doc).transform()
         node = doc.childNodes[0]
         for tagname, attr in kajiki.markup_template.QDIRECTIVES:
             if node.tagName == 'div':
@@ -66,10 +70,8 @@ class TestExpand(TestCase):
             node = node.childNodes[0]
 
 
-def perform(source, expected_output, context=dict(name='Rick'),
-            mode='xml', is_fragment=True, cdata_scripts=True):
-    tpl = XMLTemplate(source, mode=mode, is_fragment=is_fragment,
-                      cdata_scripts=cdata_scripts)
+def perform(source, expected_output, context=dict(name='Rick'), **options):
+    tpl = XMLTemplate(source, **options)
     try:
         rsp = tpl(context).render()
         assert isinstance(rsp, str), 'render() must return a unicode string.'
@@ -101,9 +103,9 @@ class TestSimple(TestCase):
     def test_script(self):
         'Always close script tags, even in xml mode.'
         source = '<html><script src="public"/></html>'
-        output = '<html><script src="public"></script>'
+        output = '<html><script src="public"></script></html>'
         perform(source, output, mode='html')
-        perform(source, output + '</html>', mode='xml')
+        perform(source, output, mode='xml')
 
     def test_script_escaping(self):
         '''In HTML script and style tags are automatically CDATA; in XML they
@@ -152,20 +154,6 @@ class TestSimple(TestCase):
         perform(src, '<script>/*<![CDATA[*//**/\n{0}/**//*]]>*/</script>'.format(
                 script), mode='xml')
 
-    def test_scripts_non_translatable(self):
-        src = '<xml><div>Hi</div><script>hello world</script><style>hello style</style></xml>'
-        doc = _Parser('<string>', src).parse()
-
-        for n in _Compiler('<string>', doc).compile():
-            text = getattr(n, 'text', '')
-            if text in ('hello world', 'hello style'):
-                self.assertFalse(isinstance(n, TranslatableTextNode))
-
-        for n in _Compiler('<string>', doc, cdata_scripts=False).compile():
-            text = getattr(n, 'text', '')
-            if text in ('hello world', 'hello style'):
-                self.assertFalse(isinstance(n, TranslatableTextNode))
-
     def test_escape_dollar(self):
         perform('<div>$$</div>', '<div>$</div>')
 
@@ -189,6 +177,16 @@ class TestSimple(TestCase):
         perform("<div>Hello, ${{'name':name}['name']}</div>",
                 '<div>Hello, Rick</div>')
 
+    def test_expr_multiline(self):
+        perform("""<div>Hello, ${{'name': 'Rick',
+                                 'age': 26}['name']}</div>""",
+                '<div>Hello, Rick</div>')
+
+    def test_expr_multiline_cdata(self):
+        perform("""<script><![CDATA[Hello, ${{'name': 'Rick',
+                                 'age': 26}['name']}]]></script>""",
+                '<script>/*<![CDATA[*/Hello, Rick/*]]>*/</script>')
+
     def test_jquery_call_is_not_expr(self):
         '''Ensure we handle '$(' as a text literal, since it cannot be a
         valid variable sequence.  This simplifies, for example,
@@ -250,6 +248,15 @@ $i is <py:switch test="i % 4">
 6 is nope</div><div>
 7 is nope</div>''')
 
+    def test_case_elem(self):
+        perform('''<div>
+    <py:switch test="True">
+      <span py:case="0 == 1">0</span>
+      <span py:case="1 == 1">1</span>
+      <span py:else="">2</span>
+    </py:switch>
+  </div>''', '<div>\n    <span>1</span>\n  </div>')
+
     def test_switch_div(self):
         try:
             tpl = perform('''
@@ -259,7 +266,7 @@ $i is <py:switch test="i % 4">
         </div>''', '<div><div>False</div></div>')
         except XMLTemplateCompileError as e:
             self.assertTrue(
-                'py:with directive can only contain py:case and py:else nodes' in str(e)
+                'py:switch directive can only contain py:case and py:else nodes' in str(e)
             )
         else:
             self.assertTrue(False, msg='Should have raised XMLTemplateParseError')
@@ -334,6 +341,10 @@ class TestWith(TestCase):
         perform('''<div py:with="a=';';b='-)'">$a$b</div>''',
                 '<div>;-)</div>')
 
+    def test_standalone(self):
+        perform('''<div><py:with vars="a=';';b='-)'">$a$b</py:with></div>''',
+                '<div>;-)</div>')
+
 
 class TestFunction(TestCase):
     def test_function(self):
@@ -351,6 +362,13 @@ class TestFunction(TestCase):
         perform('<div><py:def function="bruhaha()"></py:def></div>',
                 '<div></div>')
 
+    def test_function_in_attr(self):
+        '''Attribute value with a function call.'''
+        perform('''<div
+><py:def function="attrtest(n, sz=16)">text/$sz/$n</py:def><img
+src="${attrtest(name)}"/></div>''',
+                '<div><img src="text/16/Rick"/></div>')
+
 
 class TestCall(TestCase):
     def test_call(self):
@@ -710,20 +728,20 @@ class TestAttributes(TestCase):
         perform(TPL, '<input type="checkbox"/>', context0, mode='xml')
         perform(TPL, '<input checked="True" type="checkbox"/>',
                 context1, mode='xml')
-        perform(TPL, '<input type="checkbox">', context0, 'html')
+        perform(TPL, '<input type="checkbox">', context0, mode='html')
         perform(TPL, '<input checked type="checkbox">',
-                context1, 'html')
-        perform(TPL, '<!DOCTYPE html><input checked type="checkbox">',
+                context1, mode='html')
+        perform(TPL, '<!DOCTYPE html>\n<input checked type="checkbox">',
                 context1, mode='html5', is_fragment=False)
         perform('<!DOCTYPE html>\n' + TPL,
-                '<!DOCTYPE html><input checked type="checkbox">',
+                '<!DOCTYPE html>\n<input checked type="checkbox">',
                 context1, mode=None, is_fragment=False)
 
     def test_xml_namespaces(self):
         '''Namespaced attributes pass through.'''
         TPL = '<p xml:lang="en">English text</p>'
         perform(TPL, TPL, mode='xml')
-        perform(TPL, TPL[:-4], mode='html')
+        perform(TPL, TPL, mode='html')
 
     def test_escape_attr_values(self):
         '''Escape static and dynamic attribute values.'''
@@ -787,5 +805,77 @@ context=dict(parrot='Bereft of life, it rests in peace'))
         assert ''.join(list(literal(markup))) == markup
 
 
+class TestTranslation(TestCase):
+    def test_scripts_non_translatable(self):
+        src = '<xml><div>Hi</div><script>hello world</script><style>hello style</style></xml>'
+        doc = _Parser('<string>', src).parse()
+
+        for n in _Compiler('<string>', doc).compile():
+            text = getattr(n, 'text', '')
+            if text in ('hello world', 'hello style'):
+                self.assertFalse(isinstance(n, TranslatableTextNode))
+
+        for n in _Compiler('<string>', doc, cdata_scripts=False).compile():
+            text = getattr(n, 'text', '')
+            if text in ('hello world', 'hello style'):
+                self.assertFalse(isinstance(n, TranslatableTextNode))
+
+    def test_extract_translate(self):
+        src = '''<xml><div>Hi</div><p>
+
+        Hello
+        World</p></xml>'''
+        expected = {
+            False: '''<xml><div>TRANSLATED(Hi)</div><p>TRANSLATED(
+
+        Hello
+        World)</p></xml>''',
+            True: '''<xml><div>TRANSLATED(Hi)</div><p>TRANSLATED(Hello
+        World)</p></xml>'''
+        }
+
+        for strip_text in (False, True):
+            # Build translation table
+            messages = {}
+            for _, _, msgid, _ in i18n.extract(BytesIO(src.encode('utf-8')), None, None, {
+                'strip_text': strip_text
+            }):
+                messages[msgid] = 'TRANSLATED(%s)' % msgid
+
+            # Provide a fake translation function
+            default_gettext = i18n.gettext
+            i18n.gettext = lambda s: messages[s]
+            try:
+                perform(src, expected[strip_text], strip_text=strip_text)
+            finally:
+                i18n.gettext = default_gettext
+
+
+class TestErrorReporting(TestCase):
+    def test_syntax_error(self):
+        for strip_text in (False, True):
+            try:
+                perform('<div py:for="i i range(1, 2)">${i}</div>', '', strip_text=strip_text)
+            except KajikiSyntaxError as exc:
+                assert '-->         for i i range(1, 2):' in str(exc), exc
+            else:
+                assert False
+
+    def test_code_error(self):
+        for strip_text in (False, True):
+            try:
+                child = FileLoader(
+                    os.path.join(os.path.dirname(__file__), 'data')
+                ).load('error.html', strip_text=strip_text)
+                child().render()
+            except ZeroDivisionError as exc:
+                import traceback, sys
+                l = traceback.format_exception(*sys.exc_info())
+                last_line = l[-2]
+                assert '${3/0}' in last_line, last_line
+            else:
+                assert False
+
+
 if __name__ == '__main__':
     main()
