@@ -28,14 +28,21 @@ impl = dom.getDOMImplementation(' ')
 
 
 def XMLTemplate(source=None, filename=None, mode=None, is_fragment=False,
-                encoding='utf-8', autoblocks=None, cdata_scripts=True):
-    """Given XML source code of a Kajiki Templates parses returns a Template class.
+                encoding='utf-8', autoblocks=None, cdata_scripts=True,
+                strip_text=False):
+    """Given XML source code of a Kajiki Templates parses and returns a template class.
 
-    The source code is parsed to its DOM representation, which is then
-    expanded to separate directives from tags and then compiled to the
-    Intermediate Representation tree. The Intermediate Representation
-    is then processed to create the Python code which defined the template
-    class of which a new instance is returned.
+    The source code is parsed to its DOM representation by :class:`._Parser`,
+    which is then expanded to separate directives from tags by :class:`._DomTransformer`
+    and then compiled to the *Intermediate Representation* tree by :class:`._Compiler`.
+
+    The *Intermediate Representation* generates the Python code
+    which creates a new :class:`kajiki.template._Template` subclass through
+    :meth:`kajiki.template.Template`.
+
+    The generated code is then executed to return the newly created class.
+
+    Calling ``.render()`` on an instance of the generate class will then render the template.
     """
     if source is None:
         with open(filename, encoding=encoding) as f:
@@ -43,10 +50,9 @@ def XMLTemplate(source=None, filename=None, mode=None, is_fragment=False,
     if filename is None:
         filename = '<string>'
     doc = _Parser(filename, source).parse()
-    expand(doc)
-    compiler = _Compiler(filename, doc, mode=mode, is_fragment=is_fragment,
-                         autoblocks=autoblocks, cdata_scripts=cdata_scripts)
-    ir_ = compiler.compile()
+    doc = _DomTransformer(doc, strip_text=strip_text).transform()
+    ir_ = _Compiler(filename, doc, mode=mode, is_fragment=is_fragment,
+                    autoblocks=autoblocks, cdata_scripts=cdata_scripts).compile()
     return template.from_ir(ir_)
 
 
@@ -59,7 +65,7 @@ def annotate(gen):
 
 
 class _Compiler(object):
-    """Compiles a DOM tree into Intermediate Representation TemplateNode.
+    """Compiles a DOM tree into Intermediate Representation :class:`kajiki.ir.TemplateNode`.
 
     Intermediate Representation is a tree of nodes that represent
     Python Code that should be generated to execute the template.
@@ -88,6 +94,26 @@ class _Compiler(object):
             self.mode = 'xml'  # by default
 
     def compile(self):
+        """Compile the document provided by :class:`._Parser`.
+
+        Returns as :class:`kajiki.ir.TemplateNode` instance representing
+        the whole tree of nodes as their intermediate representation.
+
+        The returned template will include at least a ``__main__``
+        function which is the document itself including a DOCTYPE and
+        any function declared through ``py:def`` or as a ``py:block``.
+
+        The ``TemplateNode`` will also include the module level
+        code specified through ``<?py %``.
+
+        If the compiled document didn't specify a DOCTYPE provides
+        one at least for HTML5.
+
+        .. note::
+            As this alters the functions and mode wide code
+            registries of the compiler ``compile`` should
+            never be called twice or might lead to unexpected results.
+        """
         body = list(self._compile_node(self.doc.firstChild))
         # Never emit doctypes on fragments
         if not self.is_fragment and not self.is_child:
@@ -98,7 +124,7 @@ class _Compiler(object):
             else:
                 dtd = None
             if dtd:
-                dtd = ir.TextNode(dtd)
+                dtd = ir.TextNode(dtd.strip()+'\n')
                 dtd.filename = self.filename
                 dtd.lineno = 1
                 body.insert(0, dtd)
@@ -136,6 +162,14 @@ class _Compiler(object):
         return True
 
     def _compile_node(self, node):
+        """Convert a DOM node to its intermediate representation.
+
+        Calls specific compile functions for special nodes and any
+        directive that was expanded by :meth:`._DomTransformer._expand_directives`.
+        For any plain XML node forward it to :meth:`._compile_xml`.
+
+        Automatically converts any ``autoblock`` node to a ``py:block`` directive.
+        """
         if isinstance(node, dom.Comment):
             return self._compile_comment(node)
         elif isinstance(node, dom.Text):
@@ -157,6 +191,22 @@ class _Compiler(object):
 
     @annotate
     def _compile_xml(self, node):
+        """Compile plain XML nodes.
+
+        When compiling a node also take care of directives that
+        only modify the node itself (``py:strip``, ``py:attrs``
+        and ``py:content``) as all directives wrapping the node
+        and its children have already been handled by :meth:`._compile_node`.
+
+        The provided intermediate representations include
+        the node itself, its attributes and its content.
+
+        Attributes of the node are handled through :class:`._TextCompiler`
+        to ensure ${expr} expressions are handled in attributes too.
+
+        In case the node has children (and no py:content)
+        compile the children too.
+        """
         content = attrs = guard = None
         if node.hasAttribute('py:strip'):
             guard = node.getAttribute('py:strip')
@@ -226,10 +276,17 @@ class _Compiler(object):
 
     @annotate
     def _compile_replace(self, node):
+        """Convert py:replace nodes to their intermediate representation."""
         yield ir.ExprNode(node.getAttribute('value'))
 
     @annotate
     def _compile_pi(self, node):
+        """Convert <?py and <?python nodes to their intermediate representation.
+
+        Any code identified by :class:`.ir.PythonNode` as ``module_level``
+        (it starts with % character) will be registered in compiler registry
+        of module wide code to be provided to be template.
+        """
         body = ir.TextNode(node.data.strip())
         node = ir.PythonNode(body)
         if node.module_level:
@@ -239,6 +296,7 @@ class _Compiler(object):
 
     @annotate
     def _compile_import(self, node):
+        """Convert py:import nodes to their intermediate representation."""
         href = node.getAttribute('href')
         if node.hasAttribute('alias'):
             yield ir.ImportNode(href, node.getAttribute('alias'))
@@ -247,6 +305,7 @@ class _Compiler(object):
 
     @annotate
     def _compile_extends(self, node):
+        """Convert py:extends nodes to their intermediate representation."""
         self.is_child = True
         href = node.getAttribute('href')
         yield ir.ExtendNode(href)
@@ -255,11 +314,17 @@ class _Compiler(object):
 
     @annotate
     def _compile_include(self, node):
+        """Convert py:include nodes to their intermediate representation."""
         href = node.getAttribute('href')
         yield ir.IncludeNode(href)
 
     @annotate
     def _compile_block(self, node):
+        """Convert py:block nodes to their intermediate representation.
+
+        Any compiled block will be registered in the compiler functions
+        registry to be provided to the template.
+        """
         fname = '_kj_block_' + node.getAttribute('name')
         decl = fname + '()'
         body = list(self._compile_nop(node))
@@ -275,6 +340,11 @@ class _Compiler(object):
 
     @annotate
     def _compile_def(self, node):
+        """Convert py:def nodes to their intermediate representation.
+
+        Any compiled definition will be registered in the compiler functions
+        registry to be provided to the template.
+        """
         old_in_def, self.in_def = self.in_def, True
         body = list(self._compile_nop(node))
         self.in_def = old_in_def
@@ -285,6 +355,7 @@ class _Compiler(object):
 
     @annotate
     def _compile_call(self, node):
+        """Convert py:call nodes to their intermediate representation."""
         if node.childNodes[0].hasAttribute('args'):
             defn = '$caller(' + node.childNodes[0].getAttribute('args') + ')'
         else:
@@ -296,6 +367,7 @@ class _Compiler(object):
 
     @annotate
     def _compile_text(self, node):
+        """Compile text nodes to their intermediate representation"""
         kwargs = {}
         if node.parentNode and node.parentNode.tagName in HTML_CDATA_TAGS:
             # script and style should always be untranslatable.
@@ -307,21 +379,25 @@ class _Compiler(object):
 
     @annotate
     def _compile_comment(self, node):
+        """Convert comments to their intermediate representation."""
         if not node.data.startswith('!'):
             yield ir.TextNode('<!-- %s -->' % node.data)
 
     @annotate
     def _compile_for(self, node):
+        """Convert py:for nodes to their intermediate representation."""
         yield ir.ForNode(node.getAttribute('each'),
                          *list(self._compile_nop(node)))
 
     @annotate
     def _compile_with(self, node):
+        """Convert py:with nodes to their intermediate representation."""
         yield ir.WithNode(node.getAttribute('vars'),
                           *list(self._compile_nop(node)))
 
     @annotate
     def _compile_switch(self, node):
+        """Convert py:switch nodes to their intermediate representation."""
         body = []
 
         # Filter out empty text nodes and report unsupported nodes
@@ -330,7 +406,7 @@ class _Compiler(object):
                 continue
             elif not isinstance(n, (ir.CaseNode, ir.ElseNode)):
                 raise XMLTemplateCompileError(
-                    'py:with directive can only contain py:case and py:else nodes '
+                    'py:switch directive can only contain py:case and py:else nodes '
                     'and cannot be placed on a tag.',
                     doc=self.doc, filename=self.filename, linen=node.lineno
                 )
@@ -340,16 +416,19 @@ class _Compiler(object):
 
     @annotate
     def _compile_case(self, node):
+        """Convert py:case nodes to their intermediate representation."""
         yield ir.CaseNode(node.getAttribute('value'),
                           *list(self._compile_nop(node)))
 
     @annotate
     def _compile_if(self, node):
+        """Convert py:if nodes to their intermediate representation."""
         yield ir.IfNode(node.getAttribute('test'),
                         *list(self._compile_nop(node)))
 
     @annotate
     def _compile_else(self, node):
+        """Convert py:else nodes to their intermediate representation."""
         if (getattr(node.parentNode, 'tagName', '') != 'py:nop' and
                 not node.parentNode.hasAttribute('py:switch') and
                 getattr(node.previousSibling, 'tagName', '') != 'py:if'):
@@ -381,10 +460,10 @@ def make_text_node(text, guard=None):
 
 
 class _TextCompiler(object):
-    '''Separates expressions such as ${some_var} from the ordinary text
-    around them in the template source and generates ExprNode instances and
-    TextNode instances accordingly.
-    '''
+    """Separates expressions such as ${some_var} from the ordinary text
+    around them in the template source and generates :class:`.ir.ExprNode`
+    instances and :class:`.ir.TextNode` instances accordingly.
+    """
     def __init__(self, filename, source, lineno,
                  node_type=make_text_node, in_html_attr=False):
         self.filename = filename
@@ -448,7 +527,9 @@ class _TextCompiler(object):
         try:
             compile(self.source[self.pos:], '', 'eval')
         except SyntaxError as se:
-            end = se.offset + self.pos
+            end = self.pos + sum([se.offset] + [len(line) + 1
+                                                for idx, line in enumerate(self.source[self.pos:].splitlines())
+                                                if idx < se.lineno - 1])
             text = self.source[self.pos:end - 1]
             self.pos = end
             return self.expr(text)
@@ -464,7 +545,7 @@ class _Parser(sax.ContentHandler):
     DTD = '<!DOCTYPE kajiki SYSTEM "kajiki.dtd">'
 
     def __init__(self, filename, source):
-        '''XML defines only a few entities; HTML defines many more.
+        """XML defines only a few entities; HTML defines many more.
         The XML parser errors out when it finds HTML entities, unless the
         template contains a reference to an external DTD (in which case
         skippedEntity() gets called, this is what we want). In other words,
@@ -475,7 +556,8 @@ class _Parser(sax.ContentHandler):
         replace it with our own; then in the compiling stage we put the
         user's doctype back in. The XML parser is thus tricked and nobody
         needs to know this implementation detail of Kajiki.
-        '''
+        """
+        sax.ContentHandler.__init__(self)
         if not isinstance(source, str):
             raise TypeError('The template source must be a unicode string.')
         self._els = []
@@ -488,6 +570,7 @@ class _Parser(sax.ContentHandler):
         self._cdata_stack = []
 
     def parse(self):
+        """Parse an XML/HTML document to its DOM representation."""
         self._parser = parser = sax.make_parser()
         parser.setFeature(sax.handler.feature_external_pes, False)
         parser.setFeature(sax.handler.feature_external_ges, False)
@@ -547,13 +630,11 @@ class _Parser(sax.ContentHandler):
         self._els[-1].appendChild(node)
 
     def skippedEntity(self, name):
-        '''Deals with an HTML entity such as &nbsp;
-        (XML itself defines very few entities.)
+        # Deals with an HTML entity such as &nbsp; (XML itself defines very few entities.)
 
-        The presence of a SYSTEM doctype makes expat say "hey, that MIGHT be
-        a valid entity, better pass it along to sax and find out!"
-        (Since expat is nonvalidating, it never reads the external doctypes.)
-        '''
+        # The presence of a SYSTEM doctype makes expat say "hey, that MIGHT be
+        # a valid entity, better pass it along to sax and find out!"
+        # (Since expat is nonvalidating, it never reads the external doctypes.)
         return self.characters(html5[name])
 
     def startElementNS(self, name, qname, attrs):  # pragma no cover
@@ -595,63 +676,146 @@ class _Parser(sax.ContentHandler):
         pass
 
 
-def expand(tree, parent=None):
-    """Expands directives attached to nodes into separate nodes.
+class _DomTransformer(object):
+    """Applies standard Kajiki transformations to a parsed document.
 
-    This will convert all instances of::
+    Given a document generated by :class:`.Parser` it applies some
+    node transformations that are necessary before applying the
+    compilation steps to achieve result we usually expect.
 
-        <div py:if="check">
-        </div>
+    This includes things like squashing consecutive text nodes
+    and expanding ``py:`` directives.
 
-    into::
-
-        <py:if test="check">
-            <div>
-            </div>
-        </py:if>
-
-    This ensures that whenever a template is processed there is no
-    different between the two formats as the Compiler will always
-    receive the latter.
+    The Transformer mutates the original document.
     """
-    if isinstance(tree, dom.Document):
-        expand(tree.firstChild, tree)
+    def __init__(self, doc, strip_text=True):
+        self._transformed = False
+        self.doc = doc
+        self._strip_text = strip_text
+
+    def transform(self):
+        """Applies all the DOM transformations to the document.
+
+        Calling this twice will do nothing as the result is persisted.
+        """
+        if self._transformed:
+            return self.doc
+
+        self.doc = self._expand_directives(self.doc)
+        self.doc = self._merge_text_nodes(self.doc)
+        if self._strip_text:
+            self.doc = self._strip_text_nodes(self.doc)
+        return self.doc
+
+    @classmethod
+    def _merge_text_nodes(cls, tree):
+        """Merges consecutive TextNodes into a single TextNode.
+
+        Nodes are replaced with a new node whose data contains the
+        concatenation of all replaced nodes data.
+        Any other node (including CDATA TextNodes) splits runs of TextNodes.
+        """
+        if isinstance(tree, dom.Document):
+            cls._merge_text_nodes(tree.firstChild)
+            return tree
+        if not isinstance(getattr(tree, 'tagName', None), basestring):
+            return tree
+
+        # Squash all successive text nodes into a single one.
+        merge_node = None
+        for child in list(tree.childNodes):
+            if isinstance(child, dom.Text) and not getattr(child, '_cdata', False):
+                if merge_node is None:
+                    merge_node = child.ownerDocument.createTextNode(child.data)
+                    merge_node.lineno = child.lineno
+                    merge_node.escaped = child.escaped
+                    tree.replaceChild(newChild=merge_node, oldChild=child)
+                else:
+                    merge_node.data = merge_node.data + child.data
+                    tree.removeChild(child)
+            else:
+                merge_node = None
+
+        # Apply squashing to all children of current node.
+        for child in tree.childNodes:
+            if not isinstance(child, dom.Text):
+                cls._merge_text_nodes(child)
+
         return tree
-    if not isinstance(getattr(tree, 'tagName', None), basestring):
+
+    @classmethod
+    def _strip_text_nodes(cls, tree):
+        """Strips empty characters in all text nodes."""
+        for child in tree.childNodes:
+            if isinstance(child, dom.Text):
+                if not getattr(child, '_cdata', False):
+                    # Move lineno forward the amount of lines we are going to strip.
+                    lstripped_data = child.data.lstrip()
+                    child.lineno += child.data[:len(child.data)-len(lstripped_data)].count('\n')
+                    child.data = child.data.strip()
+            else:
+                cls._strip_text_nodes(child)
+        return tree
+
+    @classmethod
+    def _expand_directives(cls, tree, parent=None):
+        """Expands directives attached to nodes into separate nodes.
+
+        This will convert all instances of::
+
+            <div py:if="check">
+            </div>
+
+        into::
+
+            <py:if test="check">
+                <div>
+                </div>
+            </py:if>
+
+        This ensures that whenever a template is processed there is no
+        different between the two formats as the Compiler will always
+        receive the latter.
+        """
+        if isinstance(tree, dom.Document):
+            cls._expand_directives(tree.firstChild, tree)
+            return tree
+        if not isinstance(getattr(tree, 'tagName', None), basestring):
+            return tree
+        if tree.tagName in QDIRECTIVES_DICT:
+            tree.setAttribute(
+                tree.tagName,
+                tree.getAttribute(QDIRECTIVES_DICT[tree.tagName]))
+            tree.tagName = 'py:nop'
+        if tree.tagName != 'py:nop' and tree.hasAttribute('py:extends'):
+            value = tree.getAttribute('py:extends')
+            el = tree.ownerDocument.createElement('py:extends')
+            el.setAttribute('href', value)
+            el.lineno = tree.lineno
+            tree.removeAttribute('py:extends')
+            tree.childNodes.insert(0, el)
+        for directive, attr in QDIRECTIVES:
+            if not tree.hasAttribute(directive):
+                continue
+            value = tree.getAttribute(directive)
+            tree.removeAttribute(directive)
+            # nsmap = (parent is not None) and parent.nsmap or tree.nsmap
+            el = tree.ownerDocument.createElement(directive)
+            el.lineno = tree.lineno
+            if attr:
+                el.setAttribute(attr, value)
+            # el.setsourceline = tree.sourceline
+            parent.replaceChild(newChild=el, oldChild=tree)
+            el.appendChild(tree)
+            cls._expand_directives(tree, el)
+            return el
+        for child in tree.childNodes:
+            cls._expand_directives(child, tree)
         return tree
-    if tree.tagName in QDIRECTIVES_DICT:
-        tree.setAttribute(
-            tree.tagName,
-            tree.getAttribute(QDIRECTIVES_DICT[tree.tagName]))
-        tree.tagName = 'py:nop'
-    if tree.tagName != 'py:nop' and tree.hasAttribute('py:extends'):
-        value = tree.getAttribute('py:extends')
-        el = tree.ownerDocument.createElement('py:extends')
-        el.setAttribute('href', value)
-        el.lineno = tree.lineno
-        tree.removeAttribute('py:extends')
-        tree.childNodes.insert(0, el)
-    for directive, attr in QDIRECTIVES:
-        if not tree.hasAttribute(directive):
-            continue
-        value = tree.getAttribute(directive)
-        tree.removeAttribute(directive)
-        # nsmap = (parent is not None) and parent.nsmap or tree.nsmap
-        el = tree.ownerDocument.createElement(directive)
-        el.lineno = tree.lineno
-        if attr:
-            el.setAttribute(attr, value)
-        # el.setsourceline = tree.sourceline
-        parent.replaceChild(newChild=el, oldChild=tree)
-        el.appendChild(tree)
-        expand(tree, el)
-        return el
-    for child in tree.childNodes:
-        expand(child, tree)
-    return tree
 
 
 class XMLTemplateError(Exception):
+    """Base class for all Parse/Compile errors."""
     def __init__(self, msg, source, filename, linen, coln):
         super(XMLTemplateError, self).__init__(
             '[%s:%s] %s\n%s' % (filename, linen, msg, self._get_source_snippet(source, linen))
@@ -683,6 +847,12 @@ class XMLTemplateError(Exception):
 
 
 class XMLTemplateCompileError(XMLTemplateError):
+    """Error for failed template constraints.
+
+    This is used to signal directives in contexts where
+    they are invalid or any kajiki template constraint
+    that fails in the provided template code.
+    """
     def __init__(self, msg, doc, filename, linen):
         super(XMLTemplateCompileError, self).__init__(
             msg, getattr(doc, '_source', ''), filename, linen, 0
@@ -690,4 +860,7 @@ class XMLTemplateCompileError(XMLTemplateError):
 
 
 class XMLTemplateParseError(XMLTemplateError):
-    pass
+    """Error while parsing template XML.
+
+    Signals an invalid XML error in the provided template code.
+    """
